@@ -1,9 +1,12 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
+ * New Zealand.
+ *
  * Authors: Daniel Lawson 
- *          Perry Lorier 
+ *          Perry Lorier
+ *          Shane Alcock 
  *          
  * All rights reserved.
  *
@@ -24,9 +27,11 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_erf.c 1332 2008-03-25 21:45:36Z spa1 $
+ * $Id: format_erf.c 1556 2010-04-14 03:26:28Z perry $
  *
  */
+
+
 #define _GNU_SOURCE
 
 #include "config.h"
@@ -35,6 +40,7 @@
 #include "libtrace_int.h"
 #include "format_helper.h"
 #include "format_erf.h"
+#include "wandio.h"
 
 #include <assert.h>
 #include <errno.h>
@@ -55,60 +61,65 @@
 #  include <sys/ioctl.h>
 #endif
 
+/* This format module deals with reading and writing ERF traces. ERF is the
+ * trace format developed by Endace for use by DAG hardware capture cards.
+ *
+ * ERF is not a live capture format. 
+ *
+ */
 
-#define COLLECTOR_PORT 3435
 
 static struct libtrace_format_t erfformat;
 
 #define DATA(x) ((struct erf_format_data_t *)x->format_data)
 #define DATAOUT(x) ((struct erf_format_data_out_t *)x->format_data)
 
-#define INPUT DATA(libtrace)->input
 #define IN_OPTIONS DATA(libtrace)->options
-#define OUTPUT DATAOUT(libtrace)->output
+#define OUTPUT DATAOUT(libtrace)
 #define OUT_OPTIONS DATAOUT(libtrace)->options
+
+/* "Global" data that is stored for each ERF input trace */
 struct erf_format_data_t {
         
-	union {
-                int fd;
-		libtrace_io_t *file;
-        } input;
-
-	
+	/* Index used for seeking within a trace */
 	struct {
-		enum { INDEX_UNKNOWN=0, INDEX_NONE, INDEX_EXISTS } exists;
-		libtrace_io_t *index;
+		/* The index itself */
+		io_t *index;
+		/* The offset of the index */
 		off_t index_len;
+		/* Indicates the existence of an index */
+		enum { INDEX_UNKNOWN=0, INDEX_NONE, INDEX_EXISTS } exists;
 	} seek;
 
+	/* Number of packets that were dropped during the capture */
+	uint64_t drops;
+
+	/* Config options for the input trace */
 	struct {
+		/* Flag indicating whether the event API should replicate the
+		 * time gaps between each packet or return a PACKET event for
+		 * each packet */
 		int real_time;
 	} options;
-	uint64_t drops;
 };
 
+/* "Global" data that is stored for each ERF output trace */
 struct erf_format_data_out_t {
-	union {
-		struct {
-			int level;
-			int fileflag;
-		} erf;
-		
-	} options;
-	
-        union {
-                int fd;
-                struct rtserver_t * rtserver;
-		libtrace_io_t *file;
-        } output;
-};
 
-/** Structure holding status information for a packet */
-typedef struct libtrace_packet_status {
-	uint8_t type;
-	uint8_t reserved;
-	uint16_t message;
-} libtrace_packet_status_t;
+	/* Config options for the output trace */
+	struct {
+		/* Compression level for the output file */
+		int level;
+		/* Compression type */
+		int compress_type;
+		/* File flags used to open the file, e.g. O_CREATE */
+		int fileflag;
+	} options;
+
+	/* The output file itself */
+	iow_t *file;
+	
+};
 
 typedef struct erf_index_t {
 	uint64_t timestamp;
@@ -116,8 +127,8 @@ typedef struct erf_index_t {
 } erf_index_t;
 
 
-/* Dag erf ether packets have a 2 byte padding before the packet
- * so that the ip header is aligned on a 32 bit boundary.
+/* Ethernet packets have a 2 byte padding before the packet
+ * so that the IP header is aligned on a 32 bit boundary.
  */
 static int erf_get_padding(const libtrace_packet_t *packet)
 {
@@ -143,12 +154,49 @@ int erf_get_framing_length(const libtrace_packet_t *packet)
 	return dag_record_size + erf_get_padding(packet);
 }
 
+/* Attempts to determine whether a given trace file is using the ERF format
+ *
+ * Returns 1 if the trace is probably ERF, 0 otherwise
+ */
+static int erf_probe_magic(io_t *io)
+{
+	char buffer[4096];
+	int len;
+	dag_record_t *erf;
+	len = wandio_peek(io, buffer, sizeof(buffer));
+	if (len < (int)dag_record_size) {
+		return 0; /* False */
+	}
+	erf = (dag_record_t *) buffer;
+	/* If the record is too short */
+	if (ntohs(erf->rlen) < dag_record_size) {
+		return 0;
+	}
+	/* There aren't any erf traces before 1995-01-01 */
+	if (bswap_le_to_host64(erf->ts) < 0x2f0539b000000000ULL) {
+		return 0;
+	}
+	/* And not pcap! */
+	if (bswap_le_to_host64(erf->ts) >>32 == 0xa1b2c3d4) {
+		return 0;
+	}
+	/* And not the other pcap! */
+	if (bswap_le_to_host64(erf->ts) >>32 == 0xd4c3b2a1) {
+		return 0;
+	}
+	/* Is this a proper typed packet */
+	if (erf->type > TYPE_AAL2) {
+		return 0;
+	}
+	/* We should put some more tests in here. */
+	/* Yeah, this is probably ERF */
+	return 1;
+}
 
 static int erf_init_input(libtrace_t *libtrace) 
 {
 	libtrace->format_data = malloc(sizeof(struct erf_format_data_t));
 	
-	INPUT.file = 0;
 	IN_OPTIONS.real_time = 0;
 	DATA(libtrace)->drops = 0;
 	
@@ -179,12 +227,12 @@ static int erf_config_input(libtrace_t *libtrace, trace_option_t option,
 
 static int erf_start_input(libtrace_t *libtrace)
 {
-	if (INPUT.file)
-		return 0; /* success */
+	if (libtrace->io)
+		return 0; /* Success -- already done. */
 
-	INPUT.file = trace_open_file(libtrace);
+	libtrace->io = trace_open_file(libtrace);
 
-	if (!INPUT.file)
+	if (!libtrace->io)
 		return -1;
 
 	DATA(libtrace)->drops = 0;
@@ -204,10 +252,10 @@ static int erf_fast_seek_start(libtrace_t *libtrace,uint64_t erfts)
 	do {
 		current=(max_off+min_off)>>2;
 
-		libtrace_io_seek(DATA(libtrace)->seek.index,
+		wandio_seek(DATA(libtrace)->seek.index,
 				(int64_t)(current*sizeof(record)),
 				SEEK_SET);
-		libtrace_io_read(DATA(libtrace)->seek.index,
+		wandio_read(DATA(libtrace)->seek.index,
 				&record,sizeof(record));
 		if (record.timestamp < erfts) {
 			min_off=current;
@@ -223,15 +271,15 @@ static int erf_fast_seek_start(libtrace_t *libtrace,uint64_t erfts)
 	 * execute more than twice.
 	 */
 	do {
-		libtrace_io_seek(DATA(libtrace)->seek.index,
+		wandio_seek(DATA(libtrace)->seek.index,
 				(int64_t)(current*sizeof(record)),SEEK_SET);
-		libtrace_io_read(DATA(libtrace)->seek.index,
+		wandio_read(DATA(libtrace)->seek.index,
 				&record,sizeof(record));
 		current--;
 	} while(record.timestamp>erfts);
 
 	/* We've found our location in the trace, now use it. */
-	libtrace_io_seek(INPUT.file,(int64_t) record.offset,SEEK_SET);
+	wandio_seek(libtrace->io,(int64_t) record.offset,SEEK_SET);
 
 	return 0; /* success */
 }
@@ -241,15 +289,16 @@ static int erf_fast_seek_start(libtrace_t *libtrace,uint64_t erfts)
  */
 static int erf_slow_seek_start(libtrace_t *libtrace,uint64_t erfts UNUSED)
 {
-	if (INPUT.file) {
-		libtrace_io_close(INPUT.file);
+	if (libtrace->io) {
+		wandio_destroy(libtrace->io);
 	}
-	INPUT.file = trace_open_file(libtrace);
-	if (!INPUT.file)
+	libtrace->io = trace_open_file(libtrace);
+	if (!libtrace->io)
 		return -1;
 	return 0;
 }
 
+/* Seek within an ERF trace based on an ERF timestamp */
 static int erf_seek_erf(libtrace_t *libtrace,uint64_t erfts)
 {
 	libtrace_packet_t *packet;
@@ -258,7 +307,7 @@ static int erf_seek_erf(libtrace_t *libtrace,uint64_t erfts)
 	if (DATA(libtrace)->seek.exists==INDEX_UNKNOWN) {
 		char buffer[PATH_MAX];
 		snprintf(buffer,sizeof(buffer),"%s.idx",libtrace->uridata);
-		DATA(libtrace)->seek.index=libtrace_io_open(buffer,"rb");
+		DATA(libtrace)->seek.index=wandio_create(buffer);
 		if (DATA(libtrace)->seek.index) {
 			DATA(libtrace)->seek.exists=INDEX_EXISTS;
 		}
@@ -289,10 +338,10 @@ static int erf_seek_erf(libtrace_t *libtrace,uint64_t erfts)
 		trace_read_packet(libtrace,packet);
 		if (trace_get_erf_timestamp(packet)==erfts)
 			break;
-		off=libtrace_io_tell(INPUT.file);
+		off=wandio_tell(libtrace->io);
 	} while(trace_get_erf_timestamp(packet)<erfts);
 
-	libtrace_io_seek(INPUT.file,off,SEEK_SET);
+	wandio_seek(libtrace->io,off,SEEK_SET);
 
 	return 0;
 }
@@ -300,22 +349,26 @@ static int erf_seek_erf(libtrace_t *libtrace,uint64_t erfts)
 static int erf_init_output(libtrace_out_t *libtrace) {
 	libtrace->format_data = malloc(sizeof(struct erf_format_data_out_t));
 
-	OUT_OPTIONS.erf.level = 0;
-	OUT_OPTIONS.erf.fileflag = O_CREAT | O_WRONLY;
-	OUTPUT.file = 0;
+	OUT_OPTIONS.level = 0;
+	OUT_OPTIONS.compress_type = TRACE_OPTION_COMPRESSTYPE_NONE;
+	OUT_OPTIONS.fileflag = O_CREAT | O_WRONLY;
+	OUTPUT->file = 0;
 
 	return 0;
 }
 
-static int erf_config_output(libtrace_out_t *libtrace, trace_option_output_t option,
-		void *value) {
+static int erf_config_output(libtrace_out_t *libtrace, 
+		trace_option_output_t option, void *value) {
 
 	switch (option) {
 		case TRACE_OPTION_OUTPUT_COMPRESS:
-			OUT_OPTIONS.erf.level = *(int*)value;
+			OUT_OPTIONS.level = *(int*)value;
+			return 0;
+		case TRACE_OPTION_OUTPUT_COMPRESSTYPE:
+			OUT_OPTIONS.compress_type = *(int*)value;
 			return 0;
 		case TRACE_OPTION_OUTPUT_FILEFLAGS:
-			OUT_OPTIONS.erf.fileflag = *(int*)value;
+			OUT_OPTIONS.fileflag = *(int*)value;
 			return 0;
 		default:
 			/* Unknown option */
@@ -328,15 +381,15 @@ static int erf_config_output(libtrace_out_t *libtrace, trace_option_output_t opt
 
 
 static int erf_fin_input(libtrace_t *libtrace) {
-	if (INPUT.file)
-		libtrace_io_close(INPUT.file);
+	if (libtrace->io)
+		wandio_destroy(libtrace->io);
 	free(libtrace->format_data);
 	return 0;
 }
 
 static int erf_fin_output(libtrace_out_t *libtrace) {
-	if (OUTPUT.file)
-		libtrace_io_close(OUTPUT.file);
+	if (OUTPUT->file)
+		wandio_wdestroy(OUTPUT->file);
 	free(libtrace->format_data);
 	return 0;
 }
@@ -363,10 +416,6 @@ static int erf_prepare_packet(libtrace_t *libtrace, libtrace_packet_t *packet,
 	erfptr = (dag_record_t *)packet->buffer;
 	if (erfptr->flags.rxerror == 1) {
 		packet->payload = NULL;
-		/* XXX: Do we want to do this? We never used to do this
-		 
-		erfptr->rlen = htons(erf_get_framing_length(packet));
-		*/
 	} else {
 		packet->payload = (char*)packet->buffer + erf_get_framing_length(packet);
 	}
@@ -407,7 +456,7 @@ static int erf_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 
 	flags |= TRACE_PREP_OWN_BUFFER;	
 	
-	if ((numbytes=libtrace_io_read(INPUT.file,
+	if ((numbytes=wandio_read(libtrace->io,
 					packet->buffer,
 					(size_t)dag_record_size)) == -1) {
 		trace_set_err(libtrace,errno,"read(%s)",
@@ -424,30 +473,37 @@ static int erf_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 	size = rlen - dag_record_size;
 
 	if (size >= LIBTRACE_PACKET_BUFSIZE) {
-		trace_set_err(libtrace, TRACE_ERR_BAD_PACKET, "Packet size %u larger than supported by libtrace - packet is probably corrupt", size);
+		trace_set_err(libtrace, TRACE_ERR_BAD_PACKET, 
+				"Packet size %u larger than supported by libtrace - packet is probably corrupt", 
+				size);
 		return -1;
 	}
 
 	/* Unknown/corrupt */
-	if (((dag_record_t *)packet->buffer)->type >= TYPE_AAL2) {
-		trace_set_err(libtrace, TRACE_ERR_BAD_PACKET, "Corrupt or Unknown ERF type");
+	if (((dag_record_t *)packet->buffer)->type >= TYPE_RAW_LINK) {
+		trace_set_err(libtrace, TRACE_ERR_BAD_PACKET, 
+				"Corrupt or Unknown ERF type");
 		return -1;
 	}
 	
 	/* read in the rest of the packet */
-	if ((numbytes=libtrace_io_read(INPUT.file,
+	if ((numbytes=wandio_read(libtrace->io,
 					buffer2,
 					(size_t)size)) != (int)size) {
 		if (numbytes==-1) {
-			trace_set_err(libtrace,errno, "read(%s)", libtrace->uridata);
+			trace_set_err(libtrace,errno, "read(%s)", 
+					libtrace->uridata);
 			return -1;
 		}
-		trace_set_err(libtrace,EIO,"Truncated packet (wanted %d, got %d)", size, numbytes);
+		trace_set_err(libtrace,EIO,
+				"Truncated packet (wanted %d, got %d)", 
+				size, numbytes);
 		/* Failed to read the full packet?  must be EOF */
 		return -1;
 	}
 	
-	if (erf_prepare_packet(libtrace, packet, packet->buffer, TRACE_RT_DATA_ERF, flags))
+	if (erf_prepare_packet(libtrace, packet, packet->buffer, 
+				TRACE_RT_DATA_ERF, flags))
 		return -1;
 	
 	return rlen;
@@ -459,7 +515,7 @@ static int erf_dump_packet(libtrace_out_t *libtrace,
 	int size;
 
 	if ((numbytes = 
-		libtrace_io_write(OUTPUT.file, 
+		wandio_wwrite(OUTPUT->file, 
 				erfptr,
 				(size_t)(dag_record_size + pad))) 
 			!= (int)(dag_record_size+pad)) {
@@ -469,7 +525,7 @@ static int erf_dump_packet(libtrace_out_t *libtrace,
 	}
 
 	size=ntohs(erfptr->rlen)-(dag_record_size+pad);
-	numbytes=libtrace_io_write(OUTPUT.file, buffer, (size_t)size);
+	numbytes=wandio_wwrite(OUTPUT->file, buffer, (size_t)size);
 	if (numbytes != size) {
 		trace_set_err_out(libtrace,errno,
 				"write(%s)",libtrace->uridata);
@@ -480,10 +536,12 @@ static int erf_dump_packet(libtrace_out_t *libtrace,
 
 static int erf_start_output(libtrace_out_t *libtrace)
 {
-	OUTPUT.file = trace_open_file_out(libtrace,
-			OUT_OPTIONS.erf.level,
-			OUT_OPTIONS.erf.fileflag);
-	if (!OUTPUT.file) {
+	OUTPUT->file = trace_open_file_out(libtrace,
+			OUT_OPTIONS.compress_type,
+			OUT_OPTIONS.level,
+			OUT_OPTIONS.fileflag);
+
+	if (!OUTPUT->file) {
 		return -1;
 	}
 	return 0;
@@ -522,7 +580,7 @@ static int erf_write_packet(libtrace_out_t *libtrace,
 	dag_record_t *dag_hdr = (dag_record_t *)packet->header;
 	void *payload = packet->payload;
 
-	assert(OUTPUT.file);
+	assert(OUTPUT->file);
 
 	if (!packet->header) {
 		/*trace_set_err_output(libtrace, TRACE_ERR_BAD_PACKET,
@@ -551,6 +609,7 @@ static int erf_write_packet(libtrace_out_t *libtrace,
 				);
 	} else {
 		dag_record_t erfhdr;
+		int rlen;
 		/* convert format - build up a new erf header */
 		/* Timestamp */
 		erfhdr.ts = bswap_host_to_le64(trace_get_erf_timestamp(packet));
@@ -573,11 +632,11 @@ static int erf_write_packet(libtrace_out_t *libtrace,
 				&& trace_get_capture_length(packet)<=65536);
 		assert(erf_get_framing_length(packet)>0 
 				&& trace_get_framing_length(packet)<=65536);
-		assert(
-			trace_get_capture_length(packet)+erf_get_framing_length(packet)>0
-		      &&trace_get_capture_length(packet)+erf_get_framing_length(packet)<=65536);
-		erfhdr.rlen = htons(trace_get_capture_length(packet) 
-			+ erf_get_framing_length(packet));
+
+		rlen = trace_get_capture_length(packet) + 
+				erf_get_framing_length(packet);
+		assert(rlen > 0 && rlen <= 65536);
+		erfhdr.rlen = htons(rlen);
 		/* loss counter. Can't do this */
 		erfhdr.lctr = 0;
 		/* Wire length, does not include padding! */
@@ -648,7 +707,7 @@ size_t erf_set_capture_length(libtrace_packet_t *packet, size_t size) {
 	dag_record_t *erfptr = 0;
 	assert(packet);
 	if(size  > trace_get_capture_length(packet)) {
-		/* can't make a packet larger */
+		/* Can't make a packet larger */
 		return trace_get_capture_length(packet);
 	}
 	/* Reset cached capture length - otherwise we will both return the
@@ -663,6 +722,8 @@ size_t erf_set_capture_length(libtrace_packet_t *packet, size_t size) {
 static struct libtrace_eventobj_t erf_event(struct libtrace_t *libtrace, struct libtrace_packet_t *packet) {
 	struct libtrace_eventobj_t event = {0,0,0.0,0};
 	
+	/* If we are being told to replay packets as fast as possible, then
+	 * we just need to read and return the next packet in the trace */
 	if (IN_OPTIONS.real_time) {
 		event.size = erf_read_packet(libtrace, packet);
 		if (event.size < 1)
@@ -672,6 +733,7 @@ static struct libtrace_eventobj_t erf_event(struct libtrace_t *libtrace, struct
 		return event;
 		
 	} else {
+		/* Otherwise, use the generic event function */
 		return trace_event_trace(libtrace, packet);
 	}
 	
@@ -685,7 +747,7 @@ static uint64_t erf_get_dropped_packets(libtrace_t *trace)
 }
 
 static void erf_help(void) {
-	printf("erf format module: $Revision: 1332 $\n");
+	printf("erf format module: $Revision: 1556 $\n");
 	printf("Supported input URIs:\n");
 	printf("\terf:/path/to/file\t(uncompressed)\n");
 	printf("\terf:/path/to/file.gz\t(gzip-compressed)\n");
@@ -701,17 +763,16 @@ static void erf_help(void) {
 	printf("\n");
 	printf("\te.g.: erf:/tmp/trace\n");
 	printf("\n");
-	printf("Supported output options:\n");
-	printf("\t-z\tSpecify the gzip compression, ranging from 0 (uncompressed) to 9 - defaults to 1\n");
-	printf("\n");
 
 	
 }
 
 static struct libtrace_format_t erfformat = {
 	"erf",
-	"$Id: format_erf.c 1332 2008-03-25 21:45:36Z spa1 $",
+	"$Id: format_erf.c 1556 2010-04-14 03:26:28Z perry $",
 	TRACE_FORMAT_ERF,
+	NULL,				/* probe filename */
+	erf_probe_magic,		/* probe magic */
 	erf_init_input,			/* init_input */	
 	erf_config_input,		/* config_input */
 	erf_start_input,		/* start_input */
@@ -730,6 +791,7 @@ static struct libtrace_format_t erfformat = {
 	erf_set_direction,		/* set_direction */
 	erf_get_erf_timestamp,		/* get_erf_timestamp */
 	NULL,				/* get_timeval */
+	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
 	erf_seek_erf,			/* seek_erf */
 	NULL,				/* seek_timeval */
