@@ -1,9 +1,12 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
+ * New Zealand.
+ *
  * Authors: Daniel Lawson 
- *          Perry Lorier 
+ *          Perry Lorier
+ *          Shane Alcock 
  *          
  * All rights reserved.
  *
@@ -24,9 +27,10 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_legacy.c 1378 2008-11-05 03:46:04Z perry $
+ * $Id: format_legacy.c 1581 2010-05-05 02:56:42Z perry $
  *
  */
+
 #define _GNU_SOURCE
 
 #include "config.h"
@@ -34,6 +38,7 @@
 #include "libtrace.h"
 #include "libtrace_int.h"
 #include "format_helper.h"
+#include "wandio.h"
 
 #include <sys/stat.h>
 #include <assert.h>
@@ -49,6 +54,25 @@
 #  include <share.h>
 #endif
 
+/* The format module deals with legacy DAG formats from older revisions of the
+ * DAG hardware and software. Aside from a few minor differences, the legacy
+ * formats are very similar so we can deal with them using the same callback
+ * functions for the most part.
+ *
+ * These formats are intended for reading old ERF traces such as the earlier
+ * Auckland traces.
+ *
+ * We definitely do not support writing using these formats - one should 
+ * convert packets to regular ERF instead before writing.
+ */
+
+/* Several formats are covered in this source file:
+ *
+ * Legacy Ethernet:	as seen in Auckland VI
+ * Legacy ATM:		as seen in Auckland II and IV
+ * Legacy PoS:		as seen in Leipzig I and II
+ * Legacy NZIX:		as seen in NZIX I
+ */
 
 /* Catch undefined O_LARGEFILE on *BSD etc */
 #ifndef O_LARGEFILE
@@ -60,22 +84,21 @@
 #endif
 
 #define DATA(x) ((struct legacy_format_data_t *)x->format_data)
-#define INPUT DATA(libtrace)->input
+
+/* Legacy NZIX timestamps are all relative to the start of the trace, so we
+ * have to remember all sorts of stuff so that we can convert them into a
+ * useful timestamp */
 
 struct legacy_format_data_t {
-	union {
-                int fd;
-		libtrace_io_t *file;
-        } input;
-	time_t starttime;	/* Used for legacy_nzix */
-	uint64_t ts_high;	/* Used for legacy_nzix */
-	uint32_t ts_old; 	/* Used for legacy_nzix */
+	time_t starttime;	/* Time that the trace file was started */
+	uint64_t ts_high;	/* The timestamp of the last packet */
+	uint32_t ts_old; 	/* The timestamp of the last packet as 
+				   reported in the NZIX header */
 };
 
 static void legacy_init_format_data(libtrace_t *libtrace) {
 	libtrace->format_data = malloc(sizeof(struct legacy_format_data_t));
 	
-	DATA(libtrace)->input.file = NULL;
 	DATA(libtrace)->ts_high = 0;
 	DATA(libtrace)->ts_old = 0;
 	DATA(libtrace)->starttime = 0;
@@ -108,13 +131,17 @@ static int erf_init_input(libtrace_t *libtrace)
 	return 0;
 }
 
+/* Takes a trace file name and determines the time that the capture began.
+ *
+ * NZIX only features relative timestamps so the trace file name is the only
+ * indication we have of where the relative timestamping begins from
+ */
 static time_t trtime(char *s) {
 	/* XXX: this function may not be particularly portable to
 	 * other platforms, e.g. *BSDs, Windows */
 	struct tm tm;
 	char *tz;
 	time_t ret;
-	static char envbuf[256];
 
 	if(sscanf(s, "%4u%2u%2u-%2u%2u%2u", &tm.tm_year, &tm.tm_mon,
 				&tm.tm_mday, &tm.tm_hour, &tm.tm_min,
@@ -148,6 +175,10 @@ static int legacynzix_init_input(libtrace_t *libtrace) {
 
 
 	legacy_init_format_data(libtrace);	
+	
+	/* Check that the filename appears to contain a suitable timestamp.
+	 * Without it, we have no way of determining the actual timestamps
+	 * for each packet */
 	if((retval = regcomp(&reg, "[0-9]{8}-[0-9]{6}", REG_EXTENDED)) != 0) {
 		trace_set_err(libtrace, errno, "Failed to compile regex");
 		return -1;
@@ -160,21 +191,22 @@ static int legacynzix_init_input(libtrace_t *libtrace) {
 	return 0;
 }
 
+/* All of the formats can be started in exactly the same way */
 static int erf_start_input(libtrace_t *libtrace)
 {
-	if (DATA(libtrace)->input.file)
-		return 0;
+	if (libtrace->io)
+		return 0; /* Already open */
 
-	DATA(libtrace)->input.file = trace_open_file(libtrace);
+	libtrace->io = trace_open_file(libtrace);
 
-	if (DATA(libtrace)->input.file)
+	if (libtrace->io)
 		return 0;
 
 	return -1;
 }
 
 static int erf_fin_input(libtrace_t *libtrace) {
-	libtrace_io_close(INPUT.file);
+	wandio_destroy(libtrace->io);
 	free(libtrace->format_data);
 	return 0;
 }
@@ -236,7 +268,7 @@ static int legacy_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 	 * or we reach the end of the file */
 	while (1) {
 	
-		if ((numbytes=libtrace_io_read(INPUT.file,
+		if ((numbytes=wandio_read(libtrace->io,
 						buffer,
 						(size_t)64)) != 64) {
 			if (numbytes < 0) {
@@ -278,7 +310,7 @@ static int legacynzix_read_packet(libtrace_t *libtrace, libtrace_packet_t *packe
 	packet->type = TRACE_RT_DATA_LEGACY_NZIX;
 	
 	while (1) {
-		if ((numbytes = libtrace_io_read(INPUT.file, buffer,
+		if ((numbytes = wandio_read(libtrace->io, buffer,
 						(size_t)68)) != 68) {
 			if (numbytes < 0) {
 				trace_set_err(libtrace,errno,"read(%s)",libtrace->uridata);
@@ -296,7 +328,7 @@ static int legacynzix_read_packet(libtrace_t *libtrace, libtrace_packet_t *packe
 		break;
 	}
 
-	/* lets move the padding so that it's in the framing header */
+	/* Lets move the padding so that it's in the framing header */
 	data_ptr = ((char *)buffer) + 12;
 	memmove(data_ptr + 2, data_ptr, 26);
 
@@ -316,9 +348,25 @@ static libtrace_linktype_t legacypos_get_link_type(
  	 * Ref: RFC 1549
  	 * 	- Perry Lorier (2008-11-04)
  	 */
+	
+	/* It can also be CHDLC, which is different again */
+	
+	/* This check matchs PPP over HDLC, a la RFC 1662 */
 	if (((char *)packet->payload)[0] == '\xFF'
 		&& ((char*)packet->payload)[1] == '\x03')
 		return TRACE_TYPE_POS;
+	
+	/* This check matches unicast CHDLC */
+	else if (((char *)packet->payload)[0] == '\x0F' &&
+		((char*)packet->payload)[1] == '\x00')
+		return TRACE_TYPE_HDLC_POS;
+	
+	/* This check matches broadcast CHDLC */
+	else if (((char *)packet->payload)[0] == '\x8F' &&
+		((char*)packet->payload)[1] == '\x00')
+		return TRACE_TYPE_HDLC_POS;
+
+	/* Otherwise just assume raw PPP (?) */
 	else 
 		return TRACE_TYPE_PPP;
 }
@@ -356,8 +404,8 @@ static int legacyatm_get_wire_length(const libtrace_packet_t *packet UNUSED) {
 }
 
 static int legacyeth_get_wire_length(const libtrace_packet_t *packet) {
-	legacy_ether_t *leth = (legacy_ether_t *)packet->header;
-	return leth->wlen;
+	legacy_ether_t *leth = (legacy_ether_t *)packet->header; 
+	return leth->wlen+4; /* +4 for FCS, wirelen is in little endian sigh. */
 }
 
 static int legacynzix_get_wire_length(const libtrace_packet_t *packet) {
@@ -397,13 +445,14 @@ static struct timeval legacynzix_get_timeval(const libtrace_packet_t *packet) {
 		
 	hdr_ts = legacy->ts;
 
-	/* seems we only need 30 bits to represent our timestamp */
+	/* Seems we only need 30 bits to represent our timestamp */
 	hdr_ts >>=2;
-	/* try a sequence number wrap-around comparison */
+	
+	/* Try a sequence number wrap-around comparison */
 	if (ts_cmp(hdr_ts, old_ts) > (UINT32_MAX / 2) )
-		new_ts += (1LL << 30); /* wraparound */
-	new_ts &= ~((1LL << 30) -1);	/* mask lower 30 bits */
-	new_ts += hdr_ts;		/* packet ts is the new 30 bits */
+		new_ts += (1LL << 30); /* Wraparound */
+	new_ts &= ~((1LL << 30) -1);	/* Mask lower 30 bits */
+	new_ts += hdr_ts;		/* Packet ts is the new 30 bits */
 	DATA(packet->trace)->ts_old = hdr_ts;
 
 	tv.tv_sec = DATA(packet->trace)->starttime + (new_ts / (1000 * 1000));
@@ -417,7 +466,7 @@ static struct timeval legacynzix_get_timeval(const libtrace_packet_t *packet) {
 }	
 
 static void legacypos_help(void) {
-	printf("legacypos format module: $Revision: 1378 $\n");
+	printf("legacypos format module: $Revision: 1581 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacypos:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacypos:/path/to/file.gz\t(gzip-compressed)\n");
@@ -428,7 +477,7 @@ static void legacypos_help(void) {
 }
 
 static void legacyatm_help(void) {
-	printf("legacyatm format module: $Revision: 1378 $\n");
+	printf("legacyatm format module: $Revision: 1581 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacyatm:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacyatm:/path/to/file.gz\t(gzip-compressed)\n");
@@ -439,7 +488,7 @@ static void legacyatm_help(void) {
 }
 
 static void legacyeth_help(void) {
-	printf("legacyeth format module: $Revision: 1378 $\n");
+	printf("legacyeth format module: $Revision: 1581 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacyeth:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacyeth:/path/to/file.gz\t(gzip-compressed)\n");
@@ -450,7 +499,7 @@ static void legacyeth_help(void) {
 }
 
 static void legacynzix_help(void) {
-	printf("legacynzix format module: $Revision: 1378 $\n");
+	printf("legacynzix format module: $Revision: 1581 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacynzix:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacynzix:/path/to/file.gz\t(gzip-compressed)\n");
@@ -462,8 +511,10 @@ static void legacynzix_help(void) {
 
 static struct libtrace_format_t legacyatm = {
 	"legacyatm",
-	"$Id: format_legacy.c 1378 2008-11-05 03:46:04Z perry $",
+	"$Id: format_legacy.c 1581 2010-05-05 02:56:42Z perry $",
 	TRACE_FORMAT_LEGACY_ATM,
+	NULL,				/* probe filename */
+	NULL,				/* probe magic */
 	erf_init_input,			/* init_input */	
 	NULL,				/* config_input */
 	erf_start_input,		/* start_input */
@@ -482,6 +533,7 @@ static struct libtrace_format_t legacyatm = {
 	NULL,				/* set_direction */
 	legacy_get_erf_timestamp,	/* get_erf_timestamp */
 	NULL,				/* get_timeval */
+	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
@@ -502,8 +554,10 @@ static struct libtrace_format_t legacyatm = {
 
 static struct libtrace_format_t legacyeth = {
 	"legacyeth",
-	"$Id: format_legacy.c 1378 2008-11-05 03:46:04Z perry $",
+	"$Id: format_legacy.c 1581 2010-05-05 02:56:42Z perry $",
 	TRACE_FORMAT_LEGACY_ETH,
+	NULL,				/* probe filename */
+	NULL,				/* probe magic */
 	erf_init_input,			/* init_input */	
 	NULL,				/* config_input */
 	erf_start_input,		/* start_input */
@@ -522,6 +576,7 @@ static struct libtrace_format_t legacyeth = {
 	NULL,				/* set_direction */
 	legacy_get_erf_timestamp,	/* get_erf_timestamp */
 	NULL,				/* get_timeval */
+	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
@@ -542,8 +597,10 @@ static struct libtrace_format_t legacyeth = {
 
 static struct libtrace_format_t legacypos = {
 	"legacypos",
-	"$Id: format_legacy.c 1378 2008-11-05 03:46:04Z perry $",
+	"$Id: format_legacy.c 1581 2010-05-05 02:56:42Z perry $",
 	TRACE_FORMAT_LEGACY_POS,
+	NULL,				/* probe filename */
+	NULL,				/* probe magic */
 	erf_init_input,			/* init_input */	
 	NULL,				/* config_input */
 	erf_start_input,		/* start_input */
@@ -562,6 +619,7 @@ static struct libtrace_format_t legacypos = {
 	NULL,				/* set_direction */
 	legacy_get_erf_timestamp,	/* get_erf_timestamp */
 	NULL,				/* get_timeval */
+	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
@@ -582,8 +640,10 @@ static struct libtrace_format_t legacypos = {
 
 static struct libtrace_format_t legacynzix = {
 	"legacynzix",
-	"$Id: format_legacy.c 1378 2008-11-05 03:46:04Z perry $",
+	"$Id: format_legacy.c 1581 2010-05-05 02:56:42Z perry $",
 	TRACE_FORMAT_LEGACY_NZIX,
+	NULL,				/* probe filename */
+	NULL,				/* probe magic */
 	legacynzix_init_input,		/* init_input */	
 	NULL,				/* config_input */
 	erf_start_input,		/* start_input */
@@ -602,6 +662,7 @@ static struct libtrace_format_t legacynzix = {
 	NULL,				/* set_direction */
 	NULL,				/* get_erf_timestamp */
 	legacynzix_get_timeval,		/* get_timeval */
+	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
