@@ -1,9 +1,12 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
+ * New Zealand.
+ *
  * Authors: Daniel Lawson 
- *          Perry Lorier 
+ *          Perry Lorier
+ *          Shane Alcock 
  *          
  * All rights reserved.
  *
@@ -24,15 +27,16 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_linux.c 1352 2008-05-08 04:39:14Z perry $
+ * $Id: format_linux.c 1624 2010-07-27 05:04:51Z salcock $
  *
  */
 
+
 #include "libtrace.h"
 #include "libtrace_int.h"
 #include "format_helper.h"
 #include "config.h"
-#include "stdlib.h"
+#include <stdlib.h>
 
 #ifdef HAVE_INTTYPES_H
 #  include <inttypes.h>
@@ -49,9 +53,20 @@
 #include <net/if.h>
 #include <sys/ioctl.h>
 #include <errno.h>
+#include <unistd.h>
 
 #include <assert.h>
 
+/* This format module deals with using the Linux Native capture format.
+ *
+ * Linux Native is a LIVE capture format.
+ *
+ * This format also supports writing which will write packets out to the 
+ * network as a form of packet replay. This should not be confused with the 
+ * RT protocol which is intended to transfer captured packet records between 
+ * RT-speaking programs.
+ */
+
 /* Declared in linux/if_arp.h but not in net/if_arp.h sigh */
 #ifndef ARPHRD_NONE
 #define ARPHRD_NONE 0xfffe
@@ -62,33 +77,72 @@ struct tpacket_stats {
 	unsigned int tp_drops;
 };
 
-struct libtrace_format_data_t {
+typedef enum { TS_NONE, TS_TIMEVAL, TS_TIMESPEC } timestamptype_t;
+
+struct linux_format_data_t {
+	/* The file descriptor being used for the capture */
 	int fd;
+	/* The snap length for the capture */
 	int snaplen;
+	/* Flag indicating whether the interface should be placed in 
+	 * promiscuous mode */
 	int promisc;
+	/* The timestamp format used by the capture */ 
+	timestamptype_t timestamptype;
+	/* A BPF filter that is applied to every captured packet */
 	libtrace_filter_t *filter;
+	/* Statistics for the capture process, e.g. dropped packet counts */
 	struct tpacket_stats stats;
+	/* Flag indicating whether the statistics are current or not */
 	int stats_valid;
 };
 
+
+/* Note that this structure is passed over the wire in rt encapsulation, and 
+ * thus we need to be careful with data sizes.  timeval's and timespec's 
+ * can also change their size on 32/64 machines.
+ */
+
+/* Format header for encapsulating packets captured using linux native */
 struct libtrace_linuxnative_header {
-	struct timeval ts;
-	int wirelen;
-	int caplen;
+	/* Timestamp of the packet, as a timeval */
+	struct {
+		uint32_t tv_sec;
+		uint32_t tv_usec;
+	} tv;
+	/* Timestamp of the packet, as a timespec */
+	struct {
+		uint32_t tv_sec;
+		uint32_t tv_nsec;
+	} ts;
+	/* The timestamp format used by the process that captured this packet */
+	uint8_t timestamptype;
+	/* Wire length */
+	uint32_t wirelen;
+	/* Capture length */
+	uint32_t caplen;
+	/* The linux native header itself */
 	struct sockaddr_ll hdr;
 };
 
-struct libtrace_linuxnative_format_data_t {
+struct linux_output_format_data_t {
+	/* The file descriptor used to write the packets */
 	int fd;
 };
 
-#define FORMAT(x) ((struct libtrace_format_data_t*)(x))
-#define DATAOUT(x) ((struct libtrace_linuxnative_format_data_t*)((x)->format_data))
+#define FORMAT(x) ((struct linux_format_data_t*)(x))
+#define DATAOUT(x) ((struct linux_output_format_data_t*)((x)->format_data))
+
+static int linuxnative_probe_filename(const char *filename)
+{
+	/* Is this an interface? */
+	return (if_nametoindex(filename) != 0);
+}
 
 static int linuxnative_init_input(libtrace_t *libtrace) 
 {
-	libtrace->format_data = (struct libtrace_format_data_t *)
-		malloc(sizeof(struct libtrace_format_data_t));
+	libtrace->format_data = (struct linux_format_data_t *)
+		malloc(sizeof(struct linux_format_data_t));
 	FORMAT(libtrace->format_data)->fd = -1;
 	FORMAT(libtrace->format_data)->promisc = -1;
 	FORMAT(libtrace->format_data)->snaplen = LIBTRACE_PACKET_BUFSIZE;
@@ -100,8 +154,8 @@ static int linuxnative_init_input(libtrace_t *libtrace)
 
 static int linuxnative_init_output(libtrace_out_t *libtrace)
 {
-	libtrace->format_data = (struct libtrace_linuxnative_format_data_t*)
-		malloc(sizeof(struct libtrace_linuxnative_format_data_t));
+	libtrace->format_data = (struct linux_output_format_data_t*)
+		malloc(sizeof(struct linux_output_format_data_t));
 	DATAOUT(libtrace)->fd = -1;
 
 	return 0;
@@ -113,13 +167,17 @@ static int linuxnative_start_input(libtrace_t *libtrace)
 	int one = 1;
 	memset(&addr,0,sizeof(addr));
 	libtrace_filter_t *filter = FORMAT(libtrace->format_data)->filter;
+	
+	/* Create a raw socket for reading packets on */
 	FORMAT(libtrace->format_data)->fd = 
 				socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
 	if (FORMAT(libtrace->format_data)->fd==-1) {
-		trace_set_err(libtrace, TRACE_ERR_INIT_FAILED, "Could not create raw socket");
+		trace_set_err(libtrace, errno, "Could not create raw socket");
 		free(libtrace->format_data);
 		return -1;
 	}
+
+	/* Bind to the capture interface */
 	addr.sll_family = AF_PACKET;
 	addr.sll_protocol = htons(ETH_P_ALL);
 	if (strlen(libtrace->uridata)) {
@@ -138,7 +196,7 @@ static int linuxnative_start_input(libtrace_t *libtrace)
 				(struct sockaddr*)&addr,
 				(socklen_t)sizeof(addr))==-1) {
 		free(libtrace->format_data);
-		trace_set_err(libtrace, TRACE_ERR_INIT_FAILED, "Failed to bind to interface %s", libtrace->uridata);
+		trace_set_err(libtrace, errno, "Failed to bind to interface %s", libtrace->uridata);
 		return -1;
 	}
 
@@ -152,7 +210,8 @@ static int linuxnative_start_input(libtrace_t *libtrace)
 		else
 			FORMAT(libtrace->format_data)->promisc=0;
 	}
-				
+	
+	/* Enable promiscuous mode, if requested */			
 	if (FORMAT(libtrace->format_data)->promisc) {
 		struct packet_mreq mreq;
 		socklen_t socklen = sizeof(mreq);
@@ -168,13 +227,28 @@ static int linuxnative_start_input(libtrace_t *libtrace)
 		}
 	}
 
+	/* Set the timestamp option on the socket - aim for the most detailed 
+	 * clock resolution possible */
+#ifdef SO_TIMESTAMPNS
+	if (setsockopt(FORMAT(libtrace->format_data)->fd,
+			SOL_SOCKET,
+			SO_TIMESTAMPNS,
+			&one,
+			(socklen_t)sizeof(one))!=-1) {
+		FORMAT(libtrace->format_data)->timestamptype = TS_TIMESPEC;
+	}
+	else
+	/* DANGER: This is a dangling else to only do the next setsockopt() if we fail the first! */
+#endif
 	if (setsockopt(FORMAT(libtrace->format_data)->fd,
 			SOL_SOCKET,
 			SO_TIMESTAMP,
 			&one,
-			(socklen_t)sizeof(one))==-1) {
-		perror("setsockopt(SO_TIMESTAMP)");
+			(socklen_t)sizeof(one))!=-1) {
+		FORMAT(libtrace->format_data)->timestamptype = TS_TIMEVAL;
 	}
+	else 
+		FORMAT(libtrace->format_data)->timestamptype = TS_NONE;
 
 	/* Push BPF filter into the kernel. At this stage we can safely assume
 	 * that the filterstring has been compiled, or the filter was supplied
@@ -232,7 +306,8 @@ static int linuxnative_fin_input(libtrace_t *libtrace)
 {
 	if (FORMAT(libtrace->format_data)->filter != NULL)
 		free(FORMAT(libtrace->format_data)->filter);
-	free(libtrace->format_data);
+	if (libtrace->format_data)
+		free(libtrace->format_data);
 	
 	return 0;
 }
@@ -245,6 +320,7 @@ static int linuxnative_fin_output(libtrace_out_t *libtrace)
 	return 0;
 }
 
+/* Compiles a libtrace BPF filter for use with a linux native socket */
 static int linuxnative_configure_bpf(libtrace_t *libtrace, 
 		libtrace_filter_t *filter) {
 #ifdef HAVE_LIBPCAP 
@@ -325,6 +401,7 @@ static int linuxnative_config_input(libtrace_t *libtrace,
 			/* No meta-data for this format */
 			break;
 		case TRACE_OPTION_EVENT_REALTIME:
+			/* Live captures are always going to be in trace time */
 			break;
 		/* Avoid default: so that future options will cause a warning
 		 * here to remind us to implement it, or flag it as
@@ -398,6 +475,11 @@ static int linuxnative_read_packet(libtrace_t *libtrace, libtrace_packet_t *pack
 			(int)LIBTRACE_PACKET_BUFSIZE-(int)sizeof(*hdr),
 			(int)FORMAT(libtrace->format_data)->snaplen);
 
+	/* Prepare the msghdr and iovec for the kernel to write the
+	 * captured packet into. The msghdr will point to the part of our
+	 * buffer reserved for sll header, while the iovec will point at
+	 * the buffer following the sll header. */
+
 	msghdr.msg_name = &hdr->hdr;
 	msghdr.msg_namelen = sizeof(struct sockaddr_ll);
 
@@ -413,12 +495,16 @@ static int linuxnative_read_packet(libtrace_t *libtrace, libtrace_packet_t *pack
 
 	hdr->wirelen = recvmsg(FORMAT(libtrace->format_data)->fd, &msghdr, 0);
 
-	if (hdr->wirelen==-1) {
+	if (hdr->wirelen==~0U) {
 		trace_set_err(libtrace,errno,"recvmsg");
 		return -1;
 	}
 
-	hdr->caplen=LIBTRACE_MIN(snaplen,hdr->wirelen);
+	hdr->caplen=LIBTRACE_MIN((unsigned int)snaplen,(unsigned int)hdr->wirelen);
+
+	/* Extract the timestamps from the msghdr and store them in our
+	 * linux native encapsulation, so that we can preserve the formatting
+	 * across multiple architectures */
 
 	for (cmsg = CMSG_FIRSTHDR(&msghdr);
 			cmsg != NULL;
@@ -426,16 +512,51 @@ static int linuxnative_read_packet(libtrace_t *libtrace, libtrace_packet_t *pack
 		if (cmsg->cmsg_level == SOL_SOCKET
 			&& cmsg->cmsg_type == SO_TIMESTAMP
 			&& cmsg->cmsg_len <= CMSG_LEN(sizeof(struct timeval))) {
-			memcpy(&hdr->ts, CMSG_DATA(cmsg),
-					sizeof(struct timeval));
+			
+			struct timeval *tv;
+			tv = (struct timeval *)CMSG_DATA(cmsg);
+			
+			
+			hdr->tv.tv_sec = tv->tv_sec;
+			hdr->tv.tv_usec = tv->tv_usec;
+			hdr->timestamptype = TS_TIMEVAL;
 			break;
+		} 
+#ifdef SO_TIMESTAMPNS
+		else if (cmsg->cmsg_level == SOL_SOCKET
+			&& cmsg->cmsg_type == SO_TIMESTAMPNS
+			&& cmsg->cmsg_len <= CMSG_LEN(sizeof(struct timespec))) {
+
+			struct timespec *tv;
+			tv = (struct timespec *)CMSG_DATA(cmsg);
+
+			hdr->ts.tv_sec = tv->tv_sec;
+			hdr->ts.tv_nsec = tv->tv_nsec;
+			hdr->timestamptype = TS_TIMESPEC;
+			break;
+		}
+#endif
+	}
+
+	/* Did we not get given a timestamp? Try to get one from the
+	 * file descriptor directly */
+	if (cmsg == NULL) {
+		struct timeval tv;
+		if (ioctl(FORMAT(libtrace->format_data)->fd, 
+				  SIOCGSTAMP,&tv)==0) {
+			hdr->tv.tv_sec = tv.tv_sec;
+			hdr->tv.tv_usec = tv.tv_usec;
+			hdr->timestamptype = TS_TIMEVAL;
+		}
+		else {
+			hdr->timestamptype = TS_NONE;
 		}
 	}
 
-	if (cmsg == NULL && ioctl(FORMAT(libtrace->format_data)->fd,
-				SIOCGSTAMP,&hdr->ts)==-1)
-		perror("ioctl(SIOCGSTAMP)");
-
+	/* Buffer contains all of our packet (including our custom header) so
+	 * we just need to get prepare_packet to set all our packet pointers
+	 * appropriately */
+	
 	if (linuxnative_prepare_packet(libtrace, packet, packet->buffer,
 				packet->type, flags))
 		return -1;
@@ -454,8 +575,10 @@ static int linuxnative_write_packet(libtrace_out_t *trace,
 	hdr.sll_hatype = 0;
 	hdr.sll_pkttype = 0;
 	hdr.sll_halen = htons(6); /* FIXME */
-	memcpy(hdr.sll_addr,packet->payload,(size_t)hdr.sll_halen);
+	memcpy(hdr.sll_addr,packet->payload,(size_t)ntohs(hdr.sll_halen));
 
+	/* This is pretty easy, just send the payload using sendto() (after
+	 * setting up the sll header properly, of course) */
 	return sendto(DATAOUT(trace)->fd,
 			packet->payload,
 			trace_get_capture_length(packet),
@@ -467,6 +590,8 @@ static int linuxnative_write_packet(libtrace_out_t *trace,
 static libtrace_linktype_t linuxnative_get_link_type(const struct libtrace_packet_t *packet) {
 	int linktype=(((struct libtrace_linuxnative_header*)(packet->buffer))
 				->hdr.sll_hatype);
+	/* Convert the ARPHRD type into an appropriate libtrace link type */
+
 	switch (linktype) {
 		case ARPHRD_ETHER:
 			return TRACE_TYPE_ETH;
@@ -511,9 +636,42 @@ static libtrace_direction_t linuxnative_set_direction(
 	}
 }
 
+static struct timespec linuxnative_get_timespec(const libtrace_packet_t *packet) 
+{
+	struct libtrace_linuxnative_header *hdr = 
+		(struct libtrace_linuxnative_header*) packet->buffer;
+	/* We have to upconvert from timeval to timespec */
+	if (hdr->timestamptype == TS_TIMEVAL) {
+		struct timespec ts;
+		ts.tv_sec = hdr->tv.tv_sec;
+		ts.tv_nsec = hdr->tv.tv_usec*1000;
+		return ts;
+	}
+	else {
+		struct timespec ts;
+		ts.tv_sec = hdr->ts.tv_sec;
+		ts.tv_nsec = hdr->ts.tv_nsec;
+		return ts;
+	}
+}
+
 static struct timeval linuxnative_get_timeval(const libtrace_packet_t *packet) 
 {
-	return ((struct libtrace_linuxnative_header*)(packet->buffer))->ts;
+	struct libtrace_linuxnative_header *hdr = 
+		(struct libtrace_linuxnative_header*) packet->buffer;
+	/* We have to downconvert from timespec to timeval */
+	if (hdr->timestamptype == TS_TIMESPEC) {
+		struct timeval tv;
+		tv.tv_sec = hdr->ts.tv_sec;
+		tv.tv_usec = hdr->ts.tv_nsec/1000;
+		return tv;
+	}
+	else {
+		struct timeval tv;
+		tv.tv_sec = hdr->tv.tv_sec;
+		tv.tv_usec = hdr->tv.tv_usec;
+		return tv;
+	}
 }
 
 static int linuxnative_get_capture_length(const libtrace_packet_t *packet)
@@ -532,6 +690,24 @@ static int linuxnative_get_framing_length(UNUSED
 	return sizeof(struct libtrace_linuxnative_header);
 }
 
+static size_t linuxnative_set_capture_length(libtrace_packet_t *packet, 
+		size_t size) {
+
+	struct libtrace_linuxnative_header *linux_hdr = NULL;
+	assert(packet);
+	if (size > trace_get_capture_length(packet)) {
+		/* We should avoid making a packet larger */
+		return trace_get_capture_length(packet);
+	}
+	
+	/* Reset the cached capture length */
+	packet->capture_length = -1;
+
+	linux_hdr = (struct libtrace_linuxnative_header *)packet->header;
+	linux_hdr->caplen = size;
+	return trace_get_capture_length(packet);
+}
+
 static int linuxnative_get_fd(const libtrace_t *trace) {
 	if (trace->format_data == NULL)
 		return -1;
@@ -545,11 +721,11 @@ static int linuxnative_get_fd(const libtrace_t *trace) {
  * this here as failing, libtrace will happily report for us that it didn't
  * filter any packets, so don't lie -- return that we don't know.
  */
-static uint64_t linuxnative_get_filtered_packets(libtrace_t *trace) {
+static uint64_t linuxnative_get_filtered_packets(libtrace_t *trace UNUSED) {
 	return UINT64_MAX;
 }
 
-/* Number of packets that past filtering */
+/* Number of packets that passed filtering */
 static uint64_t linuxnative_get_captured_packets(libtrace_t *trace) {
 	if (trace->format_data == NULL)
 		return UINT64_MAX;
@@ -600,19 +776,21 @@ static uint64_t linuxnative_get_dropped_packets(libtrace_t *trace) {
 }
 
 static void linuxnative_help(void) {
-	printf("linuxnative format module: $Revision: 1352 $\n");
+	printf("linuxnative format module: $Revision: 1624 $\n");
 	printf("Supported input URIs:\n");
-	printf("\tint:\n");
+	printf("\tint:eth0\n");
 	printf("\n");
 	printf("Supported output URIs:\n");
-	printf("\tnone\n");
+	printf("\tint:eth0\n");
 	printf("\n");
 	return;
 }
 static struct libtrace_format_t linuxnative = {
 	"int",
-	"$Id: format_linux.c 1352 2008-05-08 04:39:14Z perry $",
+	"$Id: format_linux.c 1624 2010-07-27 05:04:51Z salcock $",
 	TRACE_FORMAT_LINUX_NATIVE,
+	linuxnative_probe_filename,	/* probe filename */
+	NULL,				/* probe magic */
 	linuxnative_init_input,	 	/* init_input */
 	linuxnative_config_input,	/* config_input */
 	linuxnative_start_input,	/* start_input */
@@ -631,6 +809,7 @@ static struct libtrace_format_t linuxnative = {
 	linuxnative_set_direction,	/* set_direction */
 	NULL,				/* get_erf_timestamp */
 	linuxnative_get_timeval,	/* get_timeval */
+	linuxnative_get_timespec,	/* get_timespec */
 	NULL,				/* get_seconds */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
@@ -638,7 +817,7 @@ static struct libtrace_format_t linuxnative = {
 	linuxnative_get_capture_length,	/* get_capture_length */
 	linuxnative_get_wire_length,	/* get_wire_length */
 	linuxnative_get_framing_length,	/* get_framing_length */
-	NULL,				/* set_capture_length */
+	linuxnative_set_capture_length,	/* set_capture_length */
 	NULL,				/* get_received_packets */
 	linuxnative_get_filtered_packets,/* get_filtered_packets */
 	linuxnative_get_dropped_packets,/* get_dropped_packets */
