@@ -1,9 +1,12 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
+ * New Zealand.
+ *
  * Authors: Daniel Lawson 
- *          Perry Lorier 
+ *          Perry Lorier
+ *          Shane Alcock 
  *          
  * All rights reserved.
  *
@@ -24,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_pcapfile.c 1370 2008-10-13 21:03:46Z perry $
+ * $Id: format_pcapfile.c 1603 2010-07-15 23:42:23Z salcock $
  *
  */
 
@@ -41,6 +44,18 @@
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <stdbool.h>
+
+/* This format module implements our own, more efficient, version of the PCAP
+ * file format. This should always be used in preference to the "pcap" format
+ * provided in format_pcap.c.
+ *
+ * This is a trace file format and does not implement any live interface 
+ * capture. This is covered by "pcapint" in format_pcap.c.
+ *
+ * This format supports both reading and writing, regardless of the version
+ * of your PCAP library.
+ */
 
 #define DATA(x) ((struct pcapfile_format_data_t*)((x)->format_data))
 #define DATAOUT(x) ((struct pcapfile_format_data_out_t*)((x)->format_data))
@@ -57,20 +72,44 @@ typedef struct pcapfile_header_t {
 } pcapfile_header_t; 
 
 struct pcapfile_format_data_t {
-	libtrace_io_t *file;
 	struct {
+		/* Indicates whether the event API should replicate the pauses
+		 * between packets */
 		int real_time;
 	} options;
+
+	/* The PCAP meta-header that should be written at the start of each
+	 * trace */
 	pcapfile_header_t header;
+	/* Indicates whether the input trace is started */
+	bool started;
 };
 
 struct pcapfile_format_data_out_t {
-	libtrace_io_t *file;
+	iow_t *file;
+	int compress_type;
 	int level;
 	int flag;
 
 };
 
+static int pcapfile_probe_magic(io_t *io)
+{
+	pcapfile_header_t header;
+	int len;
+	len = wandio_peek(io, &header, sizeof(header));
+	/* Is this long enough? */
+	if (len < (int)sizeof(header)) {
+		return 0;
+	}
+	/* Pcap magic? */
+	if (header.magic_number == 0xa1b2c3d4 || header.magic_number == 0xd4c3b2a1) {
+		return 1;
+	}
+	/* Nope, not pcap */
+	return 0;
+}
+
 static int pcapfile_init_input(libtrace_t *libtrace) {
 	libtrace->format_data = malloc(sizeof(struct pcapfile_format_data_t));
 
@@ -79,8 +118,8 @@ static int pcapfile_init_input(libtrace_t *libtrace) {
 		return -1;
 	}
 
-	DATA(libtrace)->file=NULL;
 	IN_OPTIONS.real_time = 0;
+	DATA(libtrace)->started = false;
 	return 0;
 }
 
@@ -89,19 +128,23 @@ static int pcapfile_init_output(libtrace_out_t *libtrace) {
 		malloc(sizeof(struct pcapfile_format_data_out_t));
 
 	DATAOUT(libtrace)->file=NULL;
+	DATAOUT(libtrace)->compress_type=TRACE_OPTION_COMPRESSTYPE_NONE;
 	DATAOUT(libtrace)->level=0;
 	DATAOUT(libtrace)->flag=O_CREAT|O_WRONLY;
 
 	return 0;
 }
 
+
 static uint16_t swaps(libtrace_t *libtrace, uint16_t num)
 {
-	/* to deal with open_dead traces that might try and use this
+	/* To deal with open_dead traces that might try and use this
 	 * if we don't have any per trace data, assume host byte order
 	 */
 	if (!DATA(libtrace))
 		return num;
+	
+	/* We can use the PCAP magic number to determine the byte order */
 	if (DATA(libtrace)->header.magic_number == 0xd4c3b2a1)
 		return byteswap16(num);
 
@@ -110,11 +153,13 @@ static uint16_t swaps(libtrace_t *libtrace, uint16_t num)
 
 static uint32_t swapl(libtrace_t *libtrace, uint32_t num)
 {
-	/* to deal with open_dead traces that might try and use this
+	/* To deal with open_dead traces that might try and use this
 	 * if we don't have any per trace data, assume host byte order
 	 */
 	if (!DATA(libtrace))
 		return num;
+	
+	/* We can use the PCAP magic number to determine the byte order */
 	if (DATA(libtrace)->header.magic_number == 0xd4c3b2a1)
 		return byteswap32(num);
 
@@ -126,27 +171,39 @@ static int pcapfile_start_input(libtrace_t *libtrace)
 {
 	int err;
 
-	if (!DATA(libtrace)->file) {
-		DATA(libtrace)->file=trace_open_file(libtrace);
+	if (!libtrace->io) {
+		libtrace->io=trace_open_file(libtrace);
+		DATA(libtrace)->started=false;
+	}
 
-		if (!DATA(libtrace)->file)
+	if (!DATA(libtrace)->started) {
+
+		if (!libtrace->io)
 			return -1;
 
-		err=libtrace_io_read(DATA(libtrace)->file,
+		err=wandio_read(libtrace->io,
 				&DATA(libtrace)->header,
 				sizeof(DATA(libtrace)->header));
 
-		if (err<1)
-			return -1;
+		DATA(libtrace)->started = true;
+		assert(sizeof(DATA(libtrace)->header) > 0);
 		
-		if (swapl(libtrace,DATA(libtrace)->header.magic_number) != 0xa1b2c3d4) {
-			trace_set_err(libtrace,TRACE_ERR_INIT_FAILED,"Not a pcap tracefile\n");
+		if (err<1) {
+
+			return -1;
+		}
+		
+		if (swapl(libtrace,DATA(libtrace)->header.magic_number) != 
+					0xa1b2c3d4) {
+			trace_set_err(libtrace,TRACE_ERR_INIT_FAILED,
+					"Not a pcap tracefile (magic=%08x)\n",swapl(libtrace,DATA(libtrace)->header.magic_number));
 			return -1; /* Not a pcap file */
 		}
 
 		if (swaps(libtrace,DATA(libtrace)->header.version_major)!=2
 			&& swaps(libtrace,DATA(libtrace)->header.version_minor)!=4) {
-			trace_set_err(libtrace,TRACE_ERR_INIT_FAILED,"Unknown pcap tracefile version %d.%d\n",
+			trace_set_err(libtrace,TRACE_ERR_INIT_FAILED,
+					"Unknown pcap tracefile version %d.%d\n",
 					swaps(libtrace,
 						DATA(libtrace)->header.version_major),
 					swaps(libtrace,
@@ -161,6 +218,10 @@ static int pcapfile_start_input(libtrace_t *libtrace)
 
 static int pcapfile_start_output(libtrace_out_t *libtrace UNUSED)
 {
+	/* We can't open the output file until we've seen the first packet.
+	 * This is because we need to know the DLT to set the "network" 
+	 * value in the meta-header */
+	
 	return 0;
 }
 
@@ -176,7 +237,7 @@ static int pcapfile_config_input(libtrace_t *libtrace,
 		case TRACE_OPTION_SNAPLEN:
 		case TRACE_OPTION_PROMISC:
 		case TRACE_OPTION_FILTER:
-			/* all these are either unsupported or handled
+			/* All these are either unsupported or handled
 			 * by trace_config */
 			break;
 	}
@@ -188,16 +249,16 @@ static int pcapfile_config_input(libtrace_t *libtrace,
 
 static int pcapfile_fin_input(libtrace_t *libtrace) 
 {
-	if (DATA(libtrace)->file)
-		libtrace_io_close(DATA(libtrace)->file);
+	if (libtrace->io)
+		wandio_destroy(libtrace->io);
 	free(libtrace->format_data);
 	return 0; /* success */
 }
 
 static int pcapfile_fin_output(libtrace_out_t *libtrace)
 {
-	if (DATA(libtrace)->file)
-		libtrace_io_close(DATA(libtrace)->file);
+	if (DATAOUT(libtrace)->file)
+		wandio_wdestroy(DATAOUT(libtrace)->file);
 	free(libtrace->format_data);
 	libtrace->format_data=NULL;
 	return 0; /* success */
@@ -211,6 +272,9 @@ static int pcapfile_config_output(libtrace_out_t *libtrace,
 		case TRACE_OPTION_OUTPUT_COMPRESS:
 			DATAOUT(libtrace)->level = *(int*)value;
 			return 0;
+		case TRACE_OPTION_OUTPUT_COMPRESSTYPE:
+			DATAOUT(libtrace)->compress_type = *(int*)value;
+			return 0;
 		case TRACE_OPTION_OUTPUT_FILEFLAGS:
 			DATAOUT(libtrace)->flag = *(int*)value;
 			return 0;
@@ -256,7 +320,8 @@ static int pcapfile_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet)
 {
 	int err;
 	uint32_t flags = 0;
-	
+	size_t bytes_to_read = 0;
+
 	assert(libtrace->format_data);
 
 	packet->type = pcap_linktype_to_rt(swapl(libtrace,
@@ -268,12 +333,10 @@ static int pcapfile_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet)
 
 	flags |= TRACE_PREP_OWN_BUFFER;
 	
-	err=libtrace_io_read(DATA(libtrace)->file,
+	err=wandio_read(libtrace->io,
 			packet->buffer,
 			sizeof(libtrace_pcapfile_pkt_hdr_t));
 
-	assert(swapl(libtrace,((libtrace_pcapfile_pkt_hdr_t*)packet->buffer)->caplen)<LIBTRACE_PACKET_BUFSIZE);
-
 	if (err<0) {
 		trace_set_err(libtrace,errno,"reading packet");
 		return -1;
@@ -283,7 +346,19 @@ static int pcapfile_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet)
 		return 0;
 	}
 
-	err=libtrace_io_read(DATA(libtrace)->file,
+	bytes_to_read = swapl(libtrace,((libtrace_pcapfile_pkt_hdr_t*)packet->buffer)->caplen);
+
+	assert(bytes_to_read < LIBTRACE_PACKET_BUFSIZE);
+
+	/* If there is no payload to read, do not ask wandio_read to try and
+	 * read zero bytes - we'll just get back a zero that we will 
+	 * misinterpret as EOF! */
+	if (bytes_to_read == 0) {
+		packet->header = packet->buffer;
+		return sizeof(libtrace_pcapfile_pkt_hdr_t);
+	}
+
+	err=wandio_read(libtrace->io,
 			(char*)packet->buffer+sizeof(libtrace_pcapfile_pkt_hdr_t),
 			(size_t)swapl(libtrace,((libtrace_pcapfile_pkt_hdr_t*)packet->buffer)->caplen)
 			);
@@ -302,8 +377,7 @@ static int pcapfile_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet)
 		return -1;
 	}
 	
-	return sizeof(libtrace_pcapfile_pkt_hdr_t)
-		+swapl(libtrace,((libtrace_pcapfile_pkt_hdr_t*)packet->buffer)->caplen);
+	return sizeof(libtrace_pcapfile_pkt_hdr_t) + bytes_to_read;
 }
 
 static int pcapfile_write_packet(libtrace_out_t *out,
@@ -321,7 +395,7 @@ static int pcapfile_write_packet(libtrace_out_t *out,
 	
 	/* Silently discard RT metadata packets and packets with an
 	 * unknown linktype. */
-	if (linktype == TRACE_TYPE_METADATA || linktype == -1) {
+	if (linktype == TRACE_TYPE_METADATA || linktype == ~0U) {
 		return 0;
 	}
 
@@ -347,10 +421,14 @@ static int pcapfile_write_packet(libtrace_out_t *out,
 		struct pcapfile_header_t pcaphdr;
 
 		DATAOUT(out)->file=trace_open_file_out(out,
+				DATAOUT(out)->compress_type,
 				DATAOUT(out)->level,
 				DATAOUT(out)->flag);
-		if (!DATAOUT(out)->file)
+
+		if (!DATAOUT(out)->file) {
+			trace_set_err_out(out,errno,"Unable to open file");
 			return -1;
+		}
 
 		pcaphdr.magic_number = 0xa1b2c3d4;
 		pcaphdr.version_major = 2;
@@ -361,7 +439,7 @@ static int pcapfile_write_packet(libtrace_out_t *out,
 		pcaphdr.network = 
 			libtrace_to_pcap_linktype(linktype);
 
-		libtrace_io_write(DATAOUT(out)->file, 
+		wandio_wwrite(DATAOUT(out)->file, 
 				&pcaphdr, sizeof(pcaphdr));
 	}
 
@@ -369,31 +447,49 @@ static int pcapfile_write_packet(libtrace_out_t *out,
 	hdr.ts_usec = tv.tv_usec;
 	hdr.caplen = trace_get_capture_length(packet);
 	assert(hdr.caplen < LIBTRACE_PACKET_BUFSIZE);
-	/* PCAP doesn't include the FCS, we do */
-	if (linktype==TRACE_TYPE_ETH)
+	/* PCAP doesn't include the FCS in its wire length value, but we do */
+	if (linktype==TRACE_TYPE_ETH) {
 		if (trace_get_wire_length(packet) >= 4) {
 			hdr.wirelen = trace_get_wire_length(packet)-4;
 		}
 		else {
 			hdr.wirelen = 0;
 		}
+	}
 	else
 		hdr.wirelen = trace_get_wire_length(packet);
 
-	assert(hdr.wirelen < LIBTRACE_PACKET_BUFSIZE);
+	/* Reason for removing this assert:
+	 *
+	 * There exist some packets, e.g. in IPLS II, where the wire length
+	 * is clearly corrupt. When converting to pcap, we *could* try to
+	 * adjust the wire length to something sane but for now, I'll just let
+	 * the broken length persist through the conversion.
+	 *
+	 * XXX Is setting the wire length to zero the best solution in such
+	 * cases?
+	 */
 
+	/* assert(hdr.wirelen < LIBTRACE_PACKET_BUFSIZE); */
 
-	numbytes=libtrace_io_write(DATAOUT(out)->file,
+	/* Ensure we have a valid capture length, especially if we're going
+	 * to "remove" the FCS from the wire length */
+	if (hdr.caplen > hdr.wirelen)
+		hdr.caplen = hdr.wirelen;
+
+	/* Write the packet header */
+	numbytes=wandio_wwrite(DATAOUT(out)->file,
 			&hdr, sizeof(hdr));
 
 	if (numbytes!=sizeof(hdr)) 
 		return -1;
 
-	ret=libtrace_io_write(DATAOUT(out)->file,
+	/* Write the rest of the packet now */
+	ret=wandio_wwrite(DATAOUT(out)->file,
 			ptr,
-			remaining);
+			hdr.caplen);
 
-	if (ret!=(int)remaining)
+	if (ret!=(int)hdr.caplen)
 		return -1;
 
 	return numbytes+ret;
@@ -402,13 +498,6 @@ static int pcapfile_write_packet(libtrace_out_t *out,
 static libtrace_linktype_t pcapfile_get_link_type(
 		const libtrace_packet_t *packet) 
 {
-#if 0
-	return pcap_linktype_to_libtrace(
-			swapl(packet->trace,
-				DATA(packet->trace)->header.network
-			     )
-			);
-#endif
 	return pcap_linktype_to_libtrace(rt_to_pcap_linktype(packet->type));
 }
 
@@ -416,6 +505,8 @@ static libtrace_direction_t pcapfile_get_direction(const libtrace_packet_t *pack
 {
 	libtrace_direction_t direction  = -1;
 	switch(pcapfile_get_link_type(packet)) {
+		/* We can only get the direction for PCAP packets that have
+		 * been encapsulated in Linux SLL or PFLOG */
 		case TRACE_TYPE_LINUX_SLL:
 		{
 			libtrace_sll_header_t *sll;
@@ -483,9 +574,12 @@ static libtrace_direction_t pcapfile_get_direction(const libtrace_packet_t *pack
 static struct timeval pcapfile_get_timeval(
 		const libtrace_packet_t *packet) 
 {
-	libtrace_pcapfile_pkt_hdr_t *hdr =
-		(libtrace_pcapfile_pkt_hdr_t*)packet->header;
+	libtrace_pcapfile_pkt_hdr_t *hdr;
 	struct timeval ts;
+	
+	assert(packet->header);
+	
+	hdr = (libtrace_pcapfile_pkt_hdr_t*)packet->header;
 	ts.tv_sec = swapl(packet->trace,hdr->ts_sec);
 	ts.tv_usec = swapl(packet->trace,hdr->ts_usec);
 	return ts;
@@ -493,15 +587,20 @@ static struct timeval pcapfile_get_timeval(
 
 
 static int pcapfile_get_capture_length(const libtrace_packet_t *packet) {
-	libtrace_pcapfile_pkt_hdr_t *pcapptr 
-		= (libtrace_pcapfile_pkt_hdr_t *)packet->header;
+	libtrace_pcapfile_pkt_hdr_t *pcapptr; 
+
+	assert(packet->header);
+	pcapptr = (libtrace_pcapfile_pkt_hdr_t *)packet->header;
 
 	return swapl(packet->trace,pcapptr->caplen);
 }
 
 static int pcapfile_get_wire_length(const libtrace_packet_t *packet) {
-	libtrace_pcapfile_pkt_hdr_t *pcapptr 
-		= (libtrace_pcapfile_pkt_hdr_t *)packet->header;
+	libtrace_pcapfile_pkt_hdr_t *pcapptr;
+	 
+	assert(packet->header); 
+
+	pcapptr	= (libtrace_pcapfile_pkt_hdr_t *)packet->header;
 	if (packet->type==pcap_linktype_to_rt(TRACE_DLT_EN10MB))
 		/* Include the missing FCS */
 		return swapl(packet->trace,pcapptr->wirelen)+4; 
@@ -528,8 +627,9 @@ static int pcapfile_get_framing_length(const libtrace_packet_t *packet UNUSED) {
 static size_t pcapfile_set_capture_length(libtrace_packet_t *packet,size_t size) {
 	libtrace_pcapfile_pkt_hdr_t *pcapptr = 0;
 	assert(packet);
+	assert(packet->header);
 	if (size > trace_get_capture_length(packet)) {
-		/* can't make a packet larger */
+		/* Can't make a packet larger */
 		return trace_get_capture_length(packet);
 	}
 	/* Reset the cached capture length */
@@ -543,6 +643,9 @@ static struct libtrace_eventobj_t pcapfile_event(libtrace_t *libtrace, libtrace_
 	
 	libtrace_eventobj_t event = {0,0,0.0,0};
 	
+	/* If we are being told to replay packets as fast as possible, then
+	 * we just need to read and return the next packet in the trace */
+
 	if (IN_OPTIONS.real_time) {
 		event.size = pcapfile_read_packet(libtrace, packet);
 		if (event.size < 1)
@@ -556,7 +659,7 @@ static struct libtrace_eventobj_t pcapfile_event(libtrace_t *libtrace, libtrace_
 }
 
 static void pcapfile_help(void) {
-	printf("pcapfile format module: $Revision: 1370 $\n");
+	printf("pcapfile format module: $Revision: 1603 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tpcapfile:/path/to/file\n");
 	printf("\tpcapfile:/path/to/file.gz\n");
@@ -567,8 +670,10 @@ static void pcapfile_help(void) {
 
 static struct libtrace_format_t pcapfile = {
 	"pcapfile",
-	"$Id: format_pcapfile.c 1370 2008-10-13 21:03:46Z perry $",
+	"$Id: format_pcapfile.c 1603 2010-07-15 23:42:23Z salcock $",
 	TRACE_FORMAT_PCAPFILE,
+	NULL,				/* probe filename */
+	pcapfile_probe_magic,		/* probe magic */
 	pcapfile_init_input,		/* init_input */
 	pcapfile_config_input,		/* config_input */
 	pcapfile_start_input,		/* start_input */
@@ -587,6 +692,7 @@ static struct libtrace_format_t pcapfile = {
 	NULL,				/* set_direction */
 	NULL,				/* get_erf_timestamp */
 	pcapfile_get_timeval,		/* get_timeval */
+	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
@@ -600,7 +706,7 @@ static struct libtrace_format_t pcapfile = {
 	NULL,				/* get_dropped_packets */
 	NULL,				/* get_captured_packets */
 	NULL,				/* get_fd */
-	trace_event_trace,		/* trace_event */
+	pcapfile_event,		/* trace_event */
 	pcapfile_help,			/* help */
 	NULL				/* next pointer */
 };
