@@ -1,8 +1,12 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
- * Authors: Perry Lorier 
+ * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
+ * New Zealand.
+ *
+ * Authors: Daniel Lawson 
+ *          Perry Lorier
+ *          Shane Alcock 
  *          
  * All rights reserved.
  *
@@ -23,9 +27,11 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_tsh.c 1375 2008-10-28 04:34:47Z perry $
+ * $Id: format_tsh.c 1611 2010-07-19 22:51:53Z salcock $
  *
  */
+
+
 #include "config.h"
 #include "common.h"
 #include "libtrace.h"
@@ -39,20 +45,19 @@
 #include <string.h>
 #include <stdlib.h>
 
+/* This format module deals with reading traces that are in the TSH format.
+ *
+ * We do not support writing TSH traces, because it's a pretty rubbish format.
+ */
+
 static struct libtrace_format_t tshformat;
 
-#define DATA(x) ((struct tsh_format_data_t *)x->format_data)
-
-struct tsh_format_data_t {
-	libtrace_io_t *file;
-};
-
 typedef struct tsh_pkt_header_t {
 	uint32_t seconds;
 	uint32_t usecs;
 } tsh_pkt_header_t;
 
-static int tsh_get_framing_length(const libtrace_packet_t *packet)
+static int tsh_get_framing_length(const libtrace_packet_t *packet UNUSED)
 {
 	return sizeof(tsh_pkt_header_t);
 }
@@ -60,30 +65,27 @@ static int tsh_get_framing_length(const libtrace_packet_t *packet)
 
 static int tsh_init_input(libtrace_t *libtrace) 
 {
-	libtrace->format_data = malloc(sizeof(struct tsh_format_data_t));
+	libtrace->format_data = NULL; /* No format data */
 	
-	DATA(libtrace)->file = 0;
-
 	return 0; /* success */
 }
 
 static int tsh_start_input(libtrace_t *libtrace)
 {
-	if (DATA(libtrace)->file)
+	if (libtrace->io)
 		return 0; /* success */
 
-	DATA(libtrace)->file = trace_open_file(libtrace);
+	libtrace->io = trace_open_file(libtrace);
 
-	if (!DATA(libtrace)->file)
+	if (!libtrace->io)
 		return -1;
 
 	return 0; /* success */
 }
 
 static int tsh_fin_input(libtrace_t *libtrace) {
-	if (DATA(libtrace)->file)
-		libtrace_io_close(DATA(libtrace)->file);
-	free(libtrace->format_data);
+	if (libtrace->io)
+		wandio_destroy(libtrace->io);
 	return 0;
 }
 
@@ -133,7 +135,7 @@ static int tsh_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 	buffer2 = packet->buffer;
 
 	/* Read the TSH header */
-	if ((numbytes=libtrace_io_read(DATA(libtrace)->file,
+	if ((numbytes=wandio_read(libtrace->io,
 					buffer2,
 					(size_t)sizeof(tsh_pkt_header_t))) == -1) {
 		trace_set_err(libtrace,errno,"read(%s)",
@@ -148,28 +150,30 @@ static int tsh_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 	buffer2 = (char*)buffer2 + numbytes;
 
 	/* Read the IP header */
-	if ((numbytes=libtrace_io_read(DATA(libtrace)->file,
+	if ((numbytes=wandio_read(libtrace->io,
 				buffer2,
-				(size_t)sizeof(libtrace_ip_t))) 
-			!= sizeof(libtrace_ip_t)) {
+				(size_t)sizeof(libtrace_ip_t)+16))  /* 16 bytes of transport header */
+			!= sizeof(libtrace_ip_t)+16) {
 		trace_set_err(libtrace,errno,"read(%s)",
 				libtrace->uridata);
 		return -1;
 	}
 
+#if 0
 	/* IP Options aren't captured in the trace, so leave room
 	 * for them, and put the transport header where it "should" be
 	 */
 	buffer2 = (char*)buffer2 + ((libtrace_ip_t*)buffer2)->ip_hl*4;
 
 	/* Read the transport header */
-	if ((numbytes=libtrace_io_read(DATA(libtrace)->file,
+	if ((numbytes=wandio_read(libtrace->io,
 				buffer2,
 				16)) != 16) {
 		trace_set_err(libtrace,errno,"read(%s)",
 				libtrace->uridata);
 		return -1;
 	}
+#endif
 	
 	if (tsh_prepare_packet(libtrace, packet, packet->buffer, packet->type, 
 				flags)) {
@@ -180,7 +184,7 @@ static int tsh_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 	return 80;
 }
 
-static libtrace_linktype_t tsh_get_link_type(const libtrace_packet_t *packet) {
+static libtrace_linktype_t tsh_get_link_type(const libtrace_packet_t *packet UNUSED) {
 	return TRACE_TYPE_NONE;
 }
 
@@ -197,19 +201,23 @@ static struct timeval tsh_get_timeval(const libtrace_packet_t *packet)
 	return tv;
 }
 
-static int tsh_get_capture_length(const libtrace_packet_t *packet) {
+static int tsh_get_capture_length(const libtrace_packet_t *packet UNUSED) {
 	/* 16 bytes transport + 24 bytes IP, and we're missing the
 	 * IP options, but we'll pretend we have them
 	 */
+#if 0
 	return 16+((libtrace_ip_t*)packet->payload)->ip_hl*4;
+#else
+	return 16+sizeof(libtrace_ip_t);
+#endif
 }
 
 static int tsh_get_wire_length(const libtrace_packet_t *packet) {
-	return ((libtrace_ip_t*)packet->payload)->ip_len;
+	return ntohs(((libtrace_ip_t*)packet->payload)->ip_len);
 }
 
 static void tsh_help(void) {
-	printf("tsh format module: $Revision: 1375 $\n");
+	printf("tsh format module: $Revision: 1611 $\n");
 	printf("Supported input URIs:\n");
 	printf("\ttsh:/path/to/file\t(uncompressed)\n");
 	printf("\ttsh:/path/to/file.gz\t(gzip-compressed)\n");
@@ -222,8 +230,10 @@ static void tsh_help(void) {
 
 static struct libtrace_format_t tshformat = {
 	"tsh",
-	"$Id: format_tsh.c 1375 2008-10-28 04:34:47Z perry $",
+	"$Id: format_tsh.c 1611 2010-07-19 22:51:53Z salcock $",
 	TRACE_FORMAT_TSH,
+	NULL,				/* probe filename */
+	NULL,				/* probe magic */
 	tsh_init_input,			/* init_input */	
 	NULL,				/* config_input */
 	tsh_start_input,		/* start_input */
@@ -242,6 +252,7 @@ static struct libtrace_format_t tshformat = {
 	NULL,				/* set_direction */
 	NULL,				/* get_erf_timestamp */
 	tsh_get_timeval,		/* get_timeval */
+	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
@@ -267,8 +278,10 @@ static struct libtrace_format_t tshformat = {
  */
 static struct libtrace_format_t frplusformat = {
 	"fr+",
-	"$Id: format_tsh.c 1375 2008-10-28 04:34:47Z perry $",
+	"$Id: format_tsh.c 1611 2010-07-19 22:51:53Z salcock $",
 	TRACE_FORMAT_TSH,
+	NULL,				/* probe filename */
+	NULL,				/* probe magic */
 	tsh_init_input,			/* init_input */	
 	NULL,				/* config_input */
 	tsh_start_input,		/* start_input */
@@ -287,6 +300,7 @@ static struct libtrace_format_t frplusformat = {
 	NULL,				/* set_direction */
 	NULL,				/* get_erf_timestamp */
 	tsh_get_timeval,		/* get_timeval */
+	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
