@@ -1,5 +1,38 @@
-#include "libtrace.h"
+/*
+ * This file is part of libtrace
+ *
+ * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
+ * New Zealand.
+ *
+ * Authors: Daniel Lawson 
+ *          Perry Lorier
+ *          Shane Alcock 
+ *          
+ * All rights reserved.
+ *
+ * This code has been developed by the University of Waikato WAND 
+ * research group. For further information please see http://www.wand.net.nz/
+ *
+ * libtrace is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * libtrace is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with libtrace; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * $Id: linktypes.c 1646 2010-08-02 03:49:15Z salcock $
+ *
+ */
+
 #include "config.h"
+#include "libtrace.h"
 
 #include "rt_protocol.h"
 #include <assert.h>
@@ -15,6 +48,10 @@
 #define ARPHRD_ETHER    1               /* Ethernet 10/100Mbps.  */
 #endif
 
+#ifndef ARPHRD_EETHER
+#define ARPHRD_EETHER    2               /* Experimental Ethernet 10/100Mbps.  */
+#endif
+
 #ifndef ARPHRD_PPP
 #define ARPHRD_PPP      512
 #endif
@@ -23,6 +60,10 @@
 #define ARPHRD_IEEE80211	801
 #endif
 
+#ifndef ARPHRD_NONE
+#define ARPHRD_NONE	0xFFFE
+#endif
+
 
 /* This file maps libtrace types to/from pcap DLT and erf types
  *
@@ -44,6 +85,7 @@ libtrace_linktype_t pcap_linktype_to_libtrace(libtrace_dlt_t linktype)
 		case TRACE_DLT_ATM_RFC1483: return TRACE_TYPE_LLCSNAP;
 		case TRACE_DLT_PPP: return TRACE_TYPE_PPP;
 		case TRACE_DLT_PPP_SERIAL: return TRACE_TYPE_POS;
+		case TRACE_DLT_C_HDLC: return TRACE_TYPE_HDLC_POS;
 		/* Unhandled */
 		case TRACE_DLT_NULL: 	/* Raw IP frame with a BSD specific
 					 * header If you want raw L3 headers
@@ -74,7 +116,7 @@ libtrace_dlt_t libtrace_to_pcap_dlt(libtrace_linktype_t type)
 		case TRACE_TYPE_POS:	return TRACE_DLT_PPP_SERIAL; 
 
 		/* Below here are unsupported conversions */
-		/* Dispite hints to the contrary, there is no DLT
+		/* Despite hints to the contrary, there is no DLT
 		 * for 'raw atm packets that happen to be missing
 		 * the HEC' or even 'raw atm packets that have a hec'.
 		 *
@@ -90,6 +132,7 @@ libtrace_dlt_t libtrace_to_pcap_dlt(libtrace_linktype_t type)
 		/* TODO: We haven't researched these yet */
 		case TRACE_TYPE_AAL5:
 		case TRACE_TYPE_METADATA:
+		case TRACE_TYPE_NONDATA:
 			break;
 	}
 	return ~0U;
@@ -152,6 +195,7 @@ uint8_t libtrace_to_erf_type(libtrace_linktype_t linktype)
 		case TRACE_TYPE_PPP:
 		case TRACE_TYPE_POS:
 		case TRACE_TYPE_METADATA:
+		case TRACE_TYPE_NONDATA:
 			break;
 	}
 	return 255;
@@ -160,9 +204,11 @@ uint8_t libtrace_to_erf_type(libtrace_linktype_t linktype)
 libtrace_linktype_t arphrd_type_to_libtrace(unsigned int arphrd) {
 	switch(arphrd) {
 		case ARPHRD_ETHER: return TRACE_TYPE_ETH;	
+		case ARPHRD_EETHER: return TRACE_TYPE_ETH;	
 		case ARPHRD_IEEE80211: return TRACE_TYPE_80211;
 		case ARPHRD_80211_RADIOTAP: return TRACE_TYPE_80211_RADIO;
 		case ARPHRD_PPP: return TRACE_TYPE_NONE;
+		case ARPHRD_NONE: return TRACE_TYPE_NONE;
 	}
 	printf("Unknown ARPHRD %08x\n",arphrd);
 	return ~0U;
@@ -178,8 +224,9 @@ unsigned int libtrace_to_arphrd_type(libtrace_linktype_t linktype) {
 	return ~0U;
 }
 
-/** Tinker with a packet
- * packets that don't support direction tagging are annoying, especially
+/** Prepends a Linux SLL header to the packet.
+ * 
+ * Packets that don't support direction tagging are annoying, especially
  * when we have direction tagging information!  So this converts the packet
  * to TRACE_TYPE_LINUX_SLL which does support direction tagging.  This is a
  * pcap style packet for the reason that it means it works with bpf filters.
@@ -250,9 +297,10 @@ void promote_packet(libtrace_packet_t *packet)
 	}
 }
 
-/* Try and simplify the packet one step, kinda the opposite to promote_packet
+/* Try and remove any extraneous encapsulation that may have been added to
+ * a packet. Effectively the opposite to promote_packet.
  *
- * returns true if demotion was possible, false if not.
+ * Returns true if demotion was possible, false if not.
  */
 bool demote_packet(libtrace_packet_t *packet)
 {
