@@ -1,9 +1,52 @@
+/*
+ * This file is part of libtrace
+ *
+ * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
+ * New Zealand.
+ *
+ * Authors: Daniel Lawson 
+ *          Perry Lorier
+ *          Shane Alcock 
+ *          
+ * All rights reserved.
+ *
+ * This code has been developed by the University of Waikato WAND 
+ * research group. For further information please see http://www.wand.net.nz/
+ *
+ * libtrace is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * libtrace is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with libtrace; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * $Id: protocols_transport.c 1522 2010-02-09 00:43:51Z salcock $
+ *
+ */
+
+
 #include "libtrace.h"
 #include "protocols.h"
 #include <assert.h>
 #include <stdlib.h>
 #include <stdio.h> // fprintf
 
+/* This file contains all the protocol decoding functions for transport layer
+ * protocols. This includes functions for access port numbers.
+ *
+ * Supported protocols include (but are not limited to):
+ * 	TCP
+ * 	UDP
+ * 	ICMP
+ */
+
 DLLEXPORT void *trace_get_transport(const libtrace_packet_t *packet, 
 		uint8_t *proto,
 		uint32_t *remaining
@@ -24,34 +67,46 @@ DLLEXPORT void *trace_get_transport(const libtrace_packet_t *packet,
 		return NULL;
 
 	switch (ethertype) {
-		case 0x0800: /* IPv4 */
+		case TRACE_ETHERTYPE_IP: /* IPv4 */
 			transport=trace_get_payload_from_ip(
 				(libtrace_ip_t*)transport, proto, remaining);
 			/* IPv6 */
-			if (transport && *proto == 41) {
+			if (transport && *proto == TRACE_IPPROTO_IPV6) {
 				transport=trace_get_payload_from_ip6(
 				 (libtrace_ip6_t*)transport, proto,remaining);
 			}
 			return transport;
-		case 0x86DD: /* IPv6 */
+		case TRACE_ETHERTYPE_IPV6: /* IPv6 */
 			return trace_get_payload_from_ip6(
 				(libtrace_ip6_t*)transport, proto, remaining);
 			
-		default:
-			fprintf(stderr,"unknown ethertype=%04x\n",ethertype);
-			*proto=0;
-			return NULL;
 	}
 
+	*proto=0;
+	return NULL;
 }
 
 DLLEXPORT libtrace_tcp_t *trace_get_tcp(libtrace_packet_t *packet) {
 	uint8_t proto;
+	uint32_t rem = 0;
 	libtrace_tcp_t *tcp;
 
-	tcp=(libtrace_tcp_t*)trace_get_transport(packet,&proto,NULL);
+	tcp=(libtrace_tcp_t*)trace_get_transport(packet,&proto,&rem);
 
-	if (!tcp || proto != 6)
+	if (!tcp || proto != TRACE_IPPROTO_TCP)
+		return NULL;
+
+	/* We should return NULL if there isn't a full TCP header, because the
+	 * caller has no way of telling how much of a TCP header we have
+	 * returned - use trace_get_transport() if you want to deal with
+	 * partial headers 
+	 *
+	 * NOTE: We're not going to insist that all the TCP options are present
+	 * as well, because lots of traces are snapped after 20 bytes of TCP
+	 * header and I don't really want to break libtrace programs that
+	 * use this function to process those traces */
+
+	if (rem < sizeof(libtrace_tcp_t))
 		return NULL;
 
 	return (libtrace_tcp_t*)tcp;
@@ -61,7 +116,7 @@ DLLEXPORT libtrace_tcp_t *trace_get_tcp_from_ip(libtrace_ip_t *ip, uint32_t *rem
 {
 	libtrace_tcp_t *tcpptr = 0;
 
-	if (ip->ip_p == 6)  {
+	if (ip->ip_p == TRACE_IPPROTO_TCP)  {
 		tcpptr = (libtrace_tcp_t *)
 			trace_get_payload_from_ip(ip, NULL, remaining);
 	}
@@ -71,11 +126,17 @@ DLLEXPORT libtrace_tcp_t *trace_get_tcp_from_ip(libtrace_ip_t *ip, uint32_t *rem
 
 DLLEXPORT libtrace_udp_t *trace_get_udp(libtrace_packet_t *packet) {
 	uint8_t proto;
+	uint32_t rem = 0;
 	libtrace_udp_t *udp;
 
-	udp=(libtrace_udp_t*)trace_get_transport(packet,&proto,NULL);
+	udp=(libtrace_udp_t*)trace_get_transport(packet,&proto,&rem);
 
-	if (!udp || proto != 17)
+	if (!udp || proto != TRACE_IPPROTO_UDP)
+		return NULL;
+
+	/* Make sure we return a full UDP header as the caller has no way of
+	 * telling how much of the packet is remaining */
+	if (rem < sizeof(libtrace_udp_t))
 		return NULL;
 
 	return udp;
@@ -85,7 +146,7 @@ DLLEXPORT libtrace_udp_t *trace_get_udp_from_ip(libtrace_ip_t *ip, uint32_t *rem
 {
 	libtrace_udp_t *udpptr = 0;
 
-	if (ip->ip_p == 17) {
+	if (ip->ip_p == TRACE_IPPROTO_UDP) {
 		udpptr = (libtrace_udp_t *)
 			trace_get_payload_from_ip(ip, NULL, remaining);
 	}
@@ -95,11 +156,17 @@ DLLEXPORT libtrace_udp_t *trace_get_udp_from_ip(libtrace_ip_t *ip, uint32_t *rem
 
 DLLEXPORT libtrace_icmp_t *trace_get_icmp(libtrace_packet_t *packet) {
 	uint8_t proto;
+	uint32_t rem = 0;
 	libtrace_icmp_t *icmp;
 
-	icmp=(libtrace_icmp_t*)trace_get_transport(packet,&proto,NULL);
+	icmp=(libtrace_icmp_t*)trace_get_transport(packet,&proto,&rem);
 
-	if (!icmp || proto != 1)
+	if (!icmp || proto != TRACE_IPPROTO_ICMP)
+		return NULL;
+
+	/* Make sure we return a full ICMP header as the caller has no way of
+	 * telling how much of the packet is remaining */
+	if (rem < sizeof(libtrace_icmp_t))
 		return NULL;
 
 	return icmp;
@@ -109,7 +176,7 @@ DLLEXPORT libtrace_icmp_t *trace_get_icmp_from_ip(libtrace_ip_t *ip, uint32_t *r
 {
 	libtrace_icmp_t *icmpptr = 0;
 
-	if (ip->ip_p == 1)  {
+	if (ip->ip_p == TRACE_IPPROTO_ICMP)  {
 		icmpptr = (libtrace_icmp_t *)trace_get_payload_from_ip(ip, 
 				NULL, remaining);
 	}
@@ -154,19 +221,24 @@ DLLEXPORT void *trace_get_payload_from_icmp(libtrace_icmp_t *icmp, uint32_t *rem
 	return (char*)icmp+sizeof(libtrace_icmp_t);
 }
 
-/* Return the client port
+/* Return the source port
  */
 DLLEXPORT uint16_t trace_get_source_port(const libtrace_packet_t *packet)
 {
 	uint32_t remaining;
+	uint8_t proto;
 	const struct ports_t *port = 
 		(const struct ports_t*)trace_get_transport((libtrace_packet_t*)packet,
-			NULL, &remaining);
+			&proto, &remaining);
 
-	/* snapped too early */
+	/* Snapped too early */
 	if (remaining<2)
 		return 0;
 
+	/* ICMP *technically* doesn't have ports */
+	if (proto == TRACE_IPPROTO_ICMP)
+		return 0;
+
 	if (port)
 		return ntohs(port->src);
 	else
@@ -177,12 +249,17 @@ DLLEXPORT uint16_t trace_get_source_port(const libtrace_packet_t *packet)
 DLLEXPORT uint16_t trace_get_destination_port(const libtrace_packet_t *packet)
 {
 	uint32_t remaining;
+	uint8_t proto;
 	struct ports_t *port = 
 		(struct ports_t*)trace_get_transport((libtrace_packet_t*)packet,
-			NULL, &remaining);
-	/* snapped to early */
+			&proto, &remaining);
+	/* Snapped too early */
 	if (remaining<4)
 		return 0;
+	
+	/* ICMP *technically* doesn't have ports */
+	if (proto == TRACE_IPPROTO_ICMP)
+		return 0;
 
 	if (port)
 		return ntohs(port->dst);
