@@ -21,6 +21,9 @@ static void usage(char *argv0)
 	"			prefix preserving\n"
 	"-p --prefix=C.I.D.R/bits Substitute the prefix of the address\n"
 	"-H --libtrace-help	Print libtrace runtime documentation\n"
+	"-z --compress-level	Compress the output trace at the specified level\n"
+	"-Z --compress-type 	Compress the output trace using the specified"
+	"			compression algorithm\n"
 	,argv0);
 	exit(1);
 }
@@ -102,13 +105,17 @@ static void encrypt_ips(struct libtrace_ip *ip,bool enc_source,bool enc_dest)
 int main(int argc, char *argv[]) 
 {
 	enum enc_type_t enc_type = ENC_NONE;
-	uint8_t *key = NULL;
+	char *key = NULL;
 	struct libtrace_t *trace = 0;
 	struct libtrace_packet_t *packet = trace_create_packet();
 	struct libtrace_out_t *writer = 0;
 	bool enc_source = false;
 	bool enc_dest 	= false;
 	char *output = 0;
+	int level = -1;
+	char *compress_type_str=NULL;
+	trace_option_compresstype_t compress_type = TRACE_OPTION_COMPRESSTYPE_NONE;
+
 
 	if (argc<2)
 		usage(argv[0]);
@@ -120,17 +127,21 @@ int main(int argc, char *argv[])
 			{ "encrypt-dest",	0, 0, 'd' },
 			{ "cryptopan",		1, 0, 'c' },
 			{ "prefix",		1, 0, 'p' },
+			{ "compress-level",	1, 0, 'z' },
+			{ "compress-type",	1, 0, 'Z' },
 			{ "libtrace-help", 	0, 0, 'H' },
 			{ NULL,			0, 0, 0   },
 		};
 
-		int c=getopt_long(argc, argv, "sc:dp:H",
+		int c=getopt_long(argc, argv, "Z:z:sc:dp:H",
 				long_options, &option_index);
 
 		if (c==-1)
 			break;
 
 		switch (c) {
+			case 'Z': compress_type_str=optarg; break;         
+			case 'z': level = atoi(optarg); break;
 			case 's': enc_source=true; break;
 			case 'd': enc_dest  =true; break;
 			case 'c': 
@@ -161,12 +172,41 @@ int main(int argc, char *argv[])
 
 	}
 
+	if (compress_type_str == NULL && level >= 0) {
+                fprintf(stderr, "Compression level set, but no compression type was defined, setting to gzip\n");
+                compress_type = TRACE_OPTION_COMPRESSTYPE_ZLIB;
+        }
+
+        else if (compress_type_str == NULL) {
+                /* If a level or type is not specified, use the "none"
+                 * compression module */
+                compress_type = TRACE_OPTION_COMPRESSTYPE_NONE;
+        }
+
+        /* I decided to be fairly generous in what I accept for the
+         * compression type string */
+        else if (strncmp(compress_type_str, "gz", 2) == 0 ||
+                        strncmp(compress_type_str, "zlib", 4) == 0) {
+                compress_type = TRACE_OPTION_COMPRESSTYPE_ZLIB;
+        } else if (strncmp(compress_type_str, "bz", 2) == 0) {
+                compress_type = TRACE_OPTION_COMPRESSTYPE_BZ2;
+        } else if (strncmp(compress_type_str, "lzo", 3) == 0) {
+                compress_type = TRACE_OPTION_COMPRESSTYPE_LZO;
+        } else if (strncmp(compress_type_str, "no", 2) == 0) {
+                compress_type = TRACE_OPTION_COMPRESSTYPE_NONE;
+        } else {
+                fprintf(stderr, "Unknown compression type: %s\n",
+                        compress_type_str);
+                return 1;
+        }
+	
+
 	enc_init(enc_type,key);
 
 	/* open input uri */
 	trace = trace_create(argv[optind]);
 	if (trace_is_err(trace)) {
-		trace_perror(trace,argv[optind]);
+		trace_perror(trace,"trace_create");
 		trace_destroy(trace);
 		return 1;
 	}
@@ -187,6 +227,29 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 	
+	/* Hopefully this will deal nicely with people who want to crank the
+	 * compression level up to 11 :) */
+	if (level > 9) {
+		fprintf(stderr, "WARNING: Compression level > 9 specified, setting to 9 instead\n");
+		level = 9;
+	}
+
+	if (level >= 0 && trace_config_output(writer, 
+			TRACE_OPTION_OUTPUT_COMPRESS, &level) == -1) {
+		trace_perror_output(writer, "Configuring compression level");
+		trace_destroy_output(writer);
+		trace_destroy(trace);
+		return 1;
+	}
+
+	if (trace_config_output(writer, TRACE_OPTION_OUTPUT_COMPRESSTYPE,
+				&compress_type) == -1) {
+		trace_perror_output(writer, "Configuring compression type");
+		trace_destroy_output(writer);
+		trace_destroy(trace);
+		return 1;
+	}
+
 	if (trace_start(trace)==-1) {
 		trace_perror(trace,"trace_start");
 		return 1;
@@ -196,6 +259,9 @@ int main(int argc, char *argv[])
 	}
 	for(;;) {
 		struct libtrace_ip *ipptr;
+		libtrace_udp_t *udp = NULL;
+		libtrace_tcp_t *tcp = NULL;
+
 		int psize;
 		psize = trace_read_packet(trace, packet);
 		if (psize == 0) {
@@ -208,8 +274,25 @@ int main(int argc, char *argv[])
 
 		ipptr = trace_get_ip(packet);
 
-		if (ipptr && (enc_source || enc_dest))
+		if (ipptr && (enc_source || enc_dest)) {
 			encrypt_ips(ipptr,enc_source,enc_dest);
+			ipptr->ip_sum = 0;
+		}
+
+		/* Replace checksums so that IP encryption cannot be
+		 * reversed */
+
+		/* XXX replace with nice use of trace_get_transport() */
+
+		udp = trace_get_udp(packet);
+		if (udp && (enc_source || enc_dest)) {
+			udp->check = 0;
+		} 
+
+		tcp = trace_get_tcp(packet);
+		if (tcp && (enc_source || enc_dest)) {
+			tcp->check = 0;
+		}
 
 		/* TODO: Encrypt IP's in ARP packets */
 
